package eu.mhsl.craftattack.spawn.appliances.privateMessage;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.chatMessages.ChatMessages;
import eu.mhsl.craftattack.spawn.appliances.privateMessage.commands.PrivateMessageCommand;
import eu.mhsl.craftattack.spawn.appliances.privateMessage.commands.PrivateReplyCommand;
import eu.mhsl.craftattack.spawn.util.LimitedSizedList;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.*;
import java.util.stream.Collectors;

public class PrivateMessage extends Appliance {
    public final int targetChangeTimeout = 30;

    private record Conversation(UUID target, Long lastSet) {}
    private final Map<Player, LimitedSizedList<Conversation>> replyMapping = new WeakHashMap<>();

    public void reply(Player sender, String message) {
        Conversation senderConversation = this.replyMapping.get(sender).getLast();
        if(senderConversation == null) throw new ApplianceCommand.Error("Du kannst nicht auf eine konversation antworten, da keine vorhanden ist.");
//        if(!senderConversation.isChecked && senderConversation.lastSet > System.currentTimeMillis() - targetChangeTimeout) {
//            throw new ApplianceCommand.Error("ziel geändedrt.......");
//        }
        Player target = Bukkit.getPlayer(senderConversation.target);
        if(target == null) throw new ApplianceCommand.Error("Der Spieler ist nicht mehr verfügbar.");
        sendWhisper(sender, new ResolvedPmUserArguments(target, message));
    }

    public void sendWhisper(Player sender, ResolvedPmUserArguments userArguments) {
//        this.replyMapping.put(
//            sender,
//            new Conversation(
//                userArguments.receiver.getUniqueId(),
//                System.currentTimeMillis()
//            )
//        );
//        this.replyMapping.put(
//            userArguments.receiver,
//            new Conversation(
//                sender.getUniqueId(),
//                System.currentTimeMillis()
//            )
//        );

        ChatMessages chatMessages = Main.instance().getAppliance(ChatMessages.class);
        Component privatePrefix = Component.text("[Privat] ", NamedTextColor.LIGHT_PURPLE);

        sender.sendMessage(
            Component.text()
                .append(privatePrefix.clickEvent(ClickEvent.suggestCommand(String.format("/msg %s ", userArguments.receiver.getName()))))
                .append(sender.displayName())
                .append(Component.text(" zu ", NamedTextColor.GRAY))
                .append(chatMessages.getReportablePlayerName(userArguments.receiver))
                .append(Component.text(" > ", NamedTextColor.GRAY))
                .append(Component.text(userArguments.message))
        );
        userArguments.receiver.sendMessage(
            Component.text()
                .append(privatePrefix.clickEvent(ClickEvent.suggestCommand(String.format("/msg %s ", sender.getName()))))
                .append(chatMessages.getReportablePlayerName(sender))
                .append(Component.text(" zu ", NamedTextColor.GRAY))
                .append(userArguments.receiver.displayName())
                .append(Component.text(" > ", NamedTextColor.GRAY))
                .append(Component.text(userArguments.message))
        );
    }

    public record ResolvedPmUserArguments(Player receiver, String message) {}
    public ResolvedPmUserArguments resolveImplicit(String[] args) {
        if(args.length < 2) throw new ApplianceCommand.Error("Es muss ein Spieler sowie eine Nachricht angegeben werden.");
        List<String> arguments = List.of(args);
        Player targetPlayer = Bukkit.getPlayer(arguments.getFirst());
        if(targetPlayer == null) throw new ApplianceCommand.Error(String.format("Der Spieler %s konnte nicht gefunden werden.", arguments.getFirst()));
        String message = arguments.stream().skip(1).collect(Collectors.joining(" "));
        return new ResolvedPmUserArguments(targetPlayer, message);
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(
            new PrivateMessageCommand(),
            new PrivateReplyCommand()
        );
    }
}
